/** @(#)Keyboard.java	1.0b 28.Januar 1998 
	* Benutzungsbeispiel:
	* Keyboard kb = new Keyboard();
	* int taste = kb.getkey();		// wartet auf Tastdruck und
	*															// gibt Taste zurck.
	* char t2 = (char) kb.getkey(); // dasselbe, in char umwandeln
	* if(kb.keypressed()) ...				// Zeichen im Tastpuffer?
	* kb.warte();										// kehrt zurck, wenn ein
	*	Achtung:											// Zeichen im Puffer ist
	* for(int i=0; i<3; i++)				// Dies wartet *nicht* drei
	*	  kb.warte();									// Tastdrcke ab;
	*										// nach dem ersten Tastdruck ist ein 
	*										// Zeichen im Tastpuffer, und warte()
  *										// gibt solange true zurck, bis es
	*										// mit getkey() abgeholt wurde
	* for(int i=0, i<3; i++)
	* 	kb.warteTaste();	// Dies hat den gewnschten Effekt.
	*											// Achtung: die drei Zeichen liegen
	*											// hinterher im Tastpuffer!
	*
	* Programme, die diese Klasse benutzen, beenden sich nicht
	* automatisch. Sie knnen per Klick auf den Beenden-Knopf
	* beendet werden oder im Programm mit System.exit(0).
	* Fr Applets oder Programe mit GUI ist die Keyboard-Klasse
	* nur sehr beschrnkt nutzbar wegen des eigenen Fensters;
	* in dem Fall empfiehlt es sich, die entsprechenden Teile
	* des Quelltextes in seine Applet-oder Fensterklasse zu
	* kopieren.
	*/
import java.awt.*;

/** Ein Taste-Objekt ist Teil einer verketteten Liste
	*	eines Tastpuffers.
	* @since		JKD1.0
*/
class Taste
{
	/** Verkettung zum nchsten Element*/
	public Taste next;
	/** Zu speichernder Tastanschlag */
	public int zeichen;
	
	/** Konstruktor */
	public Taste(int z)
	{
		zeichen = z;
	}
	
}


/** Die Tastpuffer-Klasse verwaltet eine verkettete
	* Liste von Taste-Objekten
	* @since		JKD1.0
*/
class Tastpuffer
{
	/** Schlangenkopf*/
	Taste anfang;
	/** Schlangenschwanz*/
	Taste ende;
	
	/** Konstruktor*/
	public Tastpuffer()
	{
		/* Im ersten Listenelement werden keine Daten
			 gespeichert; daher Initialisierung mit ungltigem
			 Wert  */
		anfang = new Taste(-1);
		// Am Anfang leere Liste */
		ende = anfang;
	}
		
	/** Fgt ein Zeichen in Tastpuffer ein 
		* @param z Einzufgendes Zeichen */
	public synchronized void rein(int z)
	{
		Taste t = new Taste(z);
		// neues Listenelement am Ende anhngen 
		ende.next = t;
		// Zeiger auf letztes Element aktualisieren
		ende = t;
	}
	
	/** entnimmt Element aus Tastpuffer */
	public synchronized int raus()
	{
		if(leer()) return -1;   // Sicherheitsprfung
		/* Erstes Kettenelement enthlt keine Daten;
		   daher einfach vergessen. Die Freigabe des
		   Speichers erledigt Javas Garbage Collector. */
			 		  
		anfang = anfang.next;		
		// Erstes Zeichen im Tastpuffer zurckgeben 
		return anfang.zeichen;
	}
	public synchronized boolean leer()
	{
		return anfang == ende;
	}
}

		
/** Die Keyboard-Klasse ffned ein Fenster, das Tastaturevents
	* auffngt und ein einen Tastpuffer schreibt. Diese lassen sich
	* per getkey() wieder auslesen 
  * @author  	Ingo Walther
  * @version 	1.0b, 28. Januar 1998
  * @since		JKD1.0	*/
public class Keyboard extends Frame
{
	/** Der Tastpuffer */
	Tastpuffer tp;
	
	/** In dem Fenster ist ein Button, mit dem man das Programm
	 * beenden kann. */
	Button beenden;
		
	
	/** Konstruktor */
	public Keyboard()
	{
		tp = new Tastpuffer();
		this.setTitle("Eingabefenster");// Fenstertitel setzen 		
		beenden = new Button("Beenden");// neuen Button mit der Aufschrift "Beenden" 		
		this.add(beenden);		// Knopf in Fenster einbinden
		this.pack();					// Fenster frs Anzeigen vorbereiten
		this.show();					// Fenster anzeigen
		
		requestFocus();				// Fenster in den Vordergrund und aktivieren
	}
	
	
	/** Ersetzt Methode von java.awt.Component. Wird u.a.bei
		* Bettigung des Beenden-Buttons aufgerufen 
		* @param e Aufgetretenes Event
		* @param what Fr diesen Fall nicht von Bedeutung */
		
	public boolean action(Event e, Object what)
	{
		if(e.target == beenden)	// Beenden-Button gedrckt?
		{
			this.dispose();				// Resourcen freigeben
			System.exit(0);				// Programm beenden
		}
		return false;						// sonst bekanntgeben, da Event
														// nicht verarbeitet wurde
	}	
	
	
	/** Ersetzt Methode von java.awt.Component. Wird bei jedem
		* Tastdruck aufgerufen, whrend das Fenster den Fokus hat. 
		* @param e Aufgetretenes Event 
		* @param taste Gedrckte Taste; lt sich in char umwandeln */
	public synchronized boolean keyDown(Event e, int taste)
	{
		// Taste wurde gedrckt(nicht losgelassen)?
		if(e.id == Event.KEY_PRESS)
		{
			tp.rein(taste);		// In Tastpuffer sichern
			/* Falls gerade die getkey()-Methode auf ein Zeichen wartet,
			   mu sie "aufgeweckt" werden */
			this.notifyAll();			
			return true;				// Event wurde bearbeitet
		}
		else return false;		// Event wurde nicht bearbeitet
	}
	
	
	/** keypressed() gibt true zurck, wenn Zeichen im Tastpuffer sind. */
	public boolean keypressed()
	{
		return !tp.leer();	
	}
	
	
	/** Wartet, bis ein Zeichen im Tastpuffer steht */
	public synchronized void warte()
	{
		try
		{
			/* Dies ist deutlich eleganter als while(!keypressed()),
			 * da keine Prozessorzeit verbraucht wird. Dies ist deutlich
			 * "multitasking-freundlicher". */
			while(tp.leer()) this.wait();
		}
		catch(InterruptedException e) // Falls beim Warten was schiefgeht
		{
			System.out.println("Tastatureingabe wurde unterbrochen!");
			this.dispose();							// Fenster zu
			System.exit(1);							// Programmende
		}
	}
	
	
	/** Wartet, bis eine weitere Taste gedrckt wurde(unabhngig vom
		* Status der Warteschlange)		*/
	public synchronized void warteTaste()
	{
		try
		{
			this.wait();
		}
		catch(InterruptedException e)
		{
			System.out.println("Tastatureingabe wurde unterbrochen!");
			this.dispose();							// Fenster zu
			System.exit(1);							// Programmende
		}
	}
	
	
	/** Gibt das erste Zeichen im Tastpuffer zurck. Ist dieser
		* leer, wird auf einen Tastdruck gewartet. */
	public int getkey() 
	{
		
		warte();					// bei Bedarf auf Zeichen warten
		return tp.raus();	// Zeichen aus Tastpuffer holen und
											// zurckgeben.
	}


	/** Leert den Tastpuffer aus */
	public synchronized void leeren()
	{
		while(!tp.leer())
			getkey();
	}

}

		
			
			
			
				
	